@@ -1,3 +1,4 @@
+from constants import *
 import optparse
 import sys
 import os
@@ -11,8 +12,14 @@ def pritunl_daemon():
         help='Path to create pid file')
     parser.add_option('-c', '--conf', type='string',
         help='Path to configuration file')
+    parser.add_option('--version', action='store_true',
+        help='Print version')
     (options, args) = parser.parse_args()
 
+    if options.version:
+        print '%s v%s' % (pritunl.__title__, pritunl.__version__)
+        sys.exit(0)
+
     if options.daemon:
         pid = os.fork()
         if pid > 0:
@@ -40,7 +47,7 @@ def pritunl_daemon():
     if options.conf:
         conf_path = options.conf
     else:
-        conf_path = '/etc/pritunl.conf'
+        conf_path = DEFAULT_CONF_PATH
 
     pritunl.app_server.conf_path = conf_path
     pritunl.app_server.run_server()
