@@ -1,6 +1,10 @@
 from constants import *
-from cache import persist_db
+from cache import cache_db, persist_db
 from config import Config
+import utils
+import flask
+import cherrypy.wsgiserver
+import cherrypy.wsgiserver.ssl_builtin
 import os
 import logging
 import signal
@@ -8,12 +12,13 @@ import time
 import json
 import urllib2
 import threading
-import hashlib
 import subprocess
+import base64
 
 logger = None
 
 class AppServer(Config):
+    # deprecated = password, dh_param_bits
     bool_options = {'debug', 'log_debug', 'auto_start_servers',
         'get_public_ip', 'inline_certs', 'ssl'}
     int_options = {'port', 'session_timeout', 'key_bits', 'dh_param_bits'}
@@ -38,43 +43,51 @@ class AppServer(Config):
         Config.__init__(self)
         self.app = None
         self.interrupt = False
+        self.public_ip = None
 
     def __getattr__(self, name):
         if name == 'web_protocol':
-            if self.debug or not self.ssl:
+            if not self.ssl:
                 return 'http'
             return 'https'
+        elif name == 'password_data':
+            if self.password[:2] == '1$':
+                pass_split = self.password.split('$')
+                return (1, pass_split[1], pass_split[2])
+            else:
+                return (0, PASSWORD_SALT_V0, self.password)
+        elif name == 'ssl':
+            if self.debug:
+                return False
         return Config.__getattr__(self, name)
 
-    def load_public_ip(self, retry=False, timeout=3):
-        if not self.get_public_ip:
-            return
-        logger.debug('Getting public ip address...')
-        try:
-            request = urllib2.Request(self.public_ip_server)
-            response = urllib2.urlopen(request, timeout=timeout)
-            self.public_ip = json.load(response)['ip']
-        except:
-            if retry:
+    def load_public_ip(self, attempts=1, timeout=5):
+        for i in xrange(attempts):
+            if not self.get_public_ip or self.public_ip:
+                return
+            if i != 0:
+                time.sleep(3)
                 logger.debug('Retrying get public ip address...')
-                time.sleep(1)
-                self.load_public_ip(timeout=timeout)
-            else:
+            logger.debug('Getting public ip address...')
+            try:
+                request = urllib2.Request(self.public_ip_server)
+                response = urllib2.urlopen(request, timeout=timeout)
+                self.public_ip = json.load(response)['ip']
+                break
+            except:
                 logger.exception('Failed to get public ip address...')
 
     def _setup_public_ip(self):
-        self.public_ip = None
         threading.Thread(target=self.load_public_ip,
-            kwargs={'retry': True, 'timeout': 10}).start()
+            kwargs={'attempts': 5}).start()
 
     def _setup_app(self):
-        import flask
         self.app = flask.Flask(APP_NAME)
         self.app.secret_key = os.urandom(32)
 
         @self.app.before_request
         def before_request():
-          flask.g.start = time.time()
+            flask.g.start = time.time()
 
         @self.app.after_request
         def after_request(response):
@@ -86,31 +99,16 @@ class AppServer(Config):
         logger = self.app.logger
 
     def auth(self, call):
-        import flask
-        from auth_token import AuthToken
         def _wrapped(*args, **kwargs):
-            auth_token = flask.request.headers.get('Auth-Token', None)
-            if auth_token:
-                auth_token = AuthToken(auth_token)
-                if not auth_token.valid:
-                    raise flask.abort(401)
-            else:
-                if 'timestamp' not in flask.session:
-                    raise flask.abort(401)
-
-                # Disable session timeout if set to 0
-                if self.session_timeout and time.time() - flask.session[
-                        'timestamp'] > self.session_timeout:
-                    flask.session.pop('timestamp', None)
-                    raise flask.abort(401)
+            if not utils.check_session():
+                raise flask.abort(401)
             return call(*args, **kwargs)
         _wrapped.__name__ = '%s_auth' % call.__name__
         return _wrapped
 
     def local_only(self, call):
-        import flask
         def _wrapped(*args, **kwargs):
-            if flask.request.remote_addr != '127.0.0.1':
+            if utils.get_remote_addr() not in ('127.0.0.1', '::1'):
                 raise flask.abort(401)
             return call(*args, **kwargs)
         _wrapped.__name__ = '%s_local_only' % call.__name__
@@ -166,6 +164,10 @@ class AppServer(Config):
     def _get_version_int(self, version):
         return int(''.join([x.zfill(2) for x in version.split('.')]))
 
+    def _get_version(self):
+        from pritunl import __version__
+        return self._get_version_int(__version__)
+
     def _get_data_version(self):
         version_path = os.path.join(self.data_path, VERSION_NAME)
         if os.path.isfile(version_path):
@@ -174,8 +176,7 @@ class AppServer(Config):
                     version_file.readlines()[0].strip())
 
     def _upgrade_db(self):
-        from pritunl import __version__
-        version = self._get_version_int(__version__)
+        version = self._get_version()
         cur_version = self._get_data_version()
 
         if cur_version and cur_version < self._get_version_int('0.10.5'):
@@ -186,8 +187,7 @@ class AppServer(Config):
                 pass
 
     def _upgrade_data(self):
-        from pritunl import __version__
-        version = self._get_version_int(__version__)
+        version = self._get_version()
         cur_version = self._get_data_version()
 
         if cur_version and cur_version < self._get_version_int('0.10.4'):
@@ -207,7 +207,24 @@ class AppServer(Config):
             for org in Organization.iter_orgs():
                 org._upgrade_0_10_5()
 
+        if cur_version and cur_version < self._get_version_int('0.10.6'):
+            logger.info('Upgrading data to v0.10.6...')
+            if self.password:
+                from cache import persist_db
+                logger.info('Upgrading config to v0.10.6...')
+                salt = base64.b64encode(PASSWORD_SALT_V0)
+                password = base64.b64encode(self.password.decode('hex'))
+                persist_db.dict_set('auth', 'password',
+                    '0$%s$%s' % (salt, password))
+                self.password = None
+                self.commit()
+
+            from server import Server
+            for server in Server.iter_servers():
+                server._upgrade_0_10_6()
+
         if cur_version != version:
+            from pritunl import __version__
             version_path = os.path.join(self.data_path, VERSION_NAME)
             with open(version_path, 'w') as version_file:
                 version_file.write('%s\n' % __version__)
@@ -218,26 +235,9 @@ class AppServer(Config):
         for org in Organization.iter_orgs():
             org._cache_users()
 
-    def _hash_password(self, password):
-        password_hash = hashlib.sha512()
-        password_hash.update(password)
-        password_hash.update(PASSWORD_SALT)
-        return password_hash.hexdigest()
-
-    def check_password(self, password_attempt):
-        if not self.password:
-            if password_attempt == DEFAULT_PASSWORD:
-                return True
-            return False
-
-        password_attempt = self._hash_password(password_attempt)
-        if password_attempt == self.password:
-            return True
-        return False
-
-    def set_password(self, password):
-        self.password = self._hash_password(password)
-        self.commit()
+        from server import Server
+        for server in Server.iter_servers():
+            server.load()
 
     def _setup_all(self):
         self._setup_app()
@@ -280,8 +280,6 @@ class AppServer(Config):
     def _run_wsgi(self):
         if self.ssl:
             self._setup_server_cert()
-        import cherrypy.wsgiserver
-        import cherrypy.wsgiserver.ssl_builtin
         from log_entry import LogEntry
         logger.info('Starting server...')
 
@@ -313,10 +311,10 @@ class AppServer(Config):
             raise
         finally:
             signal.signal(signal.SIGINT, signal.SIG_IGN)
-            self.interrupt = True
             logger.info('Stopping server...')
-            server.stop()
             LogEntry(message='Web server stopped.')
+            self.interrupt = True
+            server.stop()
 
     def _run_wsgi_debug(self):
         from log_entry import LogEntry
@@ -336,9 +334,9 @@ class AppServer(Config):
             raise
         finally:
             signal.signal(signal.SIGINT, signal.SIG_IGN)
-            self.interrupt = True
             logger.info('Stopping debug server...')
             LogEntry(message='Web server stopped.')
+            self.interrupt = True
 
     def _run_server(self):
         from log_entry import LogEntry
