@@ -4,7 +4,6 @@ APP_NAME = 'pritunl'
 APP_NAME_FORMATED = 'Pritunl'
 CONF_FILENAME = '%s.conf' % APP_NAME
 
-
 SAVED = 'saved'
 UNSAVED = 'unsaved'
 
@@ -12,13 +11,15 @@ START = 'start'
 STOP = 'stop'
 RESTART = 'restart'
 
-NAME_SAFE_CHARS = ['-', '_', '@', '.']
+NAME_SAFE_CHARS = {'-', '=', '_', '@', '.', ':', '/'}
 
 KEY_LINK_TIMEOUT = 86400
 DEFAULT_PRIMARY_INTERFACE = 'eth0'
 DEFAULT_SESSION_TIMEOUT = 86400
+DEFAULT_USERNAME = 'admin'
 DEFAULT_PASSWORD = 'admin'
-PASSWORD_SALT = '2511cebca93d028393735637bbc8029207731fcf'
+PASSWORD_SALT_V0 = '2511cebca93d028393735637bbc8029207731fcf'
+PASSWORD_LEN_LIMIT = 128
 DEFAULT_CONF_PATH = '/etc/pritunl.conf'
 DEFAULT_DB_PATH = '/var/lib/pritunl/pritunl.db'
 DEFAULT_WWW_PATH = '/usr/share/pritunl/www'
@@ -35,9 +36,11 @@ SHORT_URL_CHARS = (string.ascii_lowercase + string.ascii_uppercase +
     string.digits).replace('l', '').replace('I', '').replace('O', '').replace(
     '0', '')
 SUB_RESPONSE_TIMEOUT = 10
-HTTP_REQUEST_TIMEOUT = 45
+HTTP_REQUEST_TIMEOUT = 10
+HTTP_COM_REQUEST_TIMEOUT = 60
 SERVER_REQUEST_QUEUE_SIZE = 512
 USER_PAGE_COUNT = 10
+NODE_SERVER_VER = 1
 IP_REGEX = r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}'
 
 INFO = 'info'
@@ -45,7 +48,6 @@ WARNING = 'warning'
 ERROR = 'error'
 
 VERSION_NAME = 'version'
-AUTH_USER_NAME = 'admin'
 ORGS_DIR = 'organizations'
 SERVERS_DIR = 'servers'
 REQS_DIR = 'reqs'
@@ -60,6 +62,8 @@ INDEX_ATTR_NAME = 'index.attr'
 SERIAL_NAME = 'serial'
 TLS_VERIFY_NAME = 'tls_verify.py'
 USER_PASS_VERIFY_NAME = 'user_pass_verify.py'
+CLIENT_CONNECT_NAME = 'client_connect.py'
+CLIENT_DISCONNECT_NAME = 'client_disconnect.py'
 OVPN_CONF_NAME = 'openvpn.conf'
 OVPN_STATUS_NAME = 'status'
 OVPN_CA_NAME = 'ca.crt'
@@ -176,9 +180,18 @@ LOCAL_NETWORK_INVALID_MSG = 'Local network address is not valid, ' + \
     'format must be "[0-255].[0-255].[0-255].[0-254]/[8-30]" such as ' + \
     '"10.0.0.0/8".'
 
+DNS_SERVER_INVALID = 'dns_server_invalid'
+DNS_SERVER_INVALID_MSG = 'DNS server is not valid, ' + \
+    'format must be "[0-255].[0-255].[0-255].[0-255]" such as ' + \
+    '"8.8.8.8".'
+
 PORT_INVALID = 'port_invalid'
 PORT_INVALID_MSG = 'Port number is not valid, must be between 1 and 65535.'
 
+DH_PARAM_BITS_INVALID = 'dh_param_bits_invalid'
+DH_PARAM_BITS_INVALID_MSG = 'DH param bits are not valid, must ' + \
+    '1024, 1536, 2048, 2048, 3072 or 4096.'
+
 INTERFACE_INVALID = 'interface_invalid'
 INTERFACE_INVALID_MSG = 'Interface is not valid, must be ' + \
     '"tun[0-64]" example "tun0".'
@@ -205,6 +218,12 @@ SERVER_NOT_OFFLINE_DETACH_ORG_MSG = 'Server must be offline to detach ' + \
 SERVER_INVALID = 'server_invalid'
 SERVER_INVALID_MSG = 'Server is not valid.'
 
+NODE_API_KEY_INVLID = 'node_api_key_invlid'
+NODE_API_KEY_INVLID_MSG = 'Node server api key is invalid.'
+
+NODE_CONNECTION_ERROR = 'node_connection_error'
+NODE_CONNECTION_ERROR_MSG = 'Failed to connect to node server.'
+
 ORG_INVALID = 'organization_invalid'
 ORG_INVALID_MSG = 'Organization is not valid.'
 
@@ -221,6 +240,8 @@ ca %s
 cert %s
 key %s
 tls-verify %s
+client-connect %s
+client-disconnect %s
 dh %s
 server %s
 ifconfig-pool-persist %s
@@ -239,6 +260,8 @@ OVPN_INLINE_SERVER_CONF = """port %s
 proto %s
 dev %s
 tls-verify %s
+client-connect %s
+client-disconnect %s
 server %s
 ifconfig-pool-persist %s
 %s
@@ -287,7 +310,7 @@ VALID_CHARS = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789='
 auth_log_path = '%s'
 def log_write(line):
     with open(auth_log_path, 'a') as auth_log_file:
-        auth_log_file.write('[TIME=%%s]%%s\\n' %% (
+        auth_log_file.write('[TLS_VERIFY][TIME=%%s]%%s\\n' %% (
             int(time.time()), line.rstrip('\\n')))
 
 try:
@@ -367,7 +390,7 @@ VALID_CHARS = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789='
 auth_log_path = '%s'
 def log_write(line):
     with open(auth_log_path, 'a') as auth_log_file:
-        auth_log_file.write('[TIME=%%s]%%s\\n' %% (
+        auth_log_file.write('[OTP_VERIFY][TIME=%%s]%%s\\n' %% (
             int(time.time()), line.rstrip('\\n')))
 
 try:
@@ -447,3 +470,164 @@ except:
 
 exit(0)
 """
+
+# Script will run in python 2 and 3
+CLIENT_CONNECT_SCRIPT = """#!/usr/bin/env python
+import os
+import sys
+import json
+import time
+import traceback
+
+VALID_CHARS = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789='
+auth_log_path = '%s'
+def log_write(line):
+    with open(auth_log_path, 'a') as auth_log_file:
+        auth_log_file.write('[CLIENT_CONNECT][TIME=%%s]%%s\\n' %% (
+            int(time.time()), line.rstrip('\\n')))
+
+try:
+    try:
+        from urllib2 import urlopen
+    except ImportError:
+        from urllib.request import urlopen
+    try:
+        from urllib2 import Request
+    except ImportError:
+        from urllib.request import Request
+    try:
+        from urllib2 import HTTPError
+    except ImportError:
+        from urllib.error import HTTPError
+    try:
+        from socket import error as SocketError
+    except ImportError:
+        SocketError = ConnectionResetError
+
+    # Get org and common_name from environ
+    tls_env = os.environ.get('tls_id_0')
+    if not tls_env:
+        log_write('[FAILED] Missing organization or user id from environ')
+        raise AttributeError('Missing organization or user id from environ')
+    tls_env = ''.join(x for x in tls_env if x in VALID_CHARS)
+    o_index = tls_env.find('O=')
+    cn_index = tls_env.find('CN=')
+    if o_index < 0 or cn_index < 0:
+        log_write('[FAILED] Missing organization or user id from environ')
+        raise AttributeError('Missing organization or user id from environ')
+    if o_index > cn_index:
+        org = tls_env[o_index + 2:]
+        common_name = tls_env[3:o_index]
+    else:
+        org = tls_env[2:cn_index]
+        common_name = tls_env[cn_index + 3:]
+    if not org or not common_name:
+        log_write('[FAILED] Missing organization or user id from environ')
+        raise AttributeError('Missing organization or user id from environ')
+
+    try:
+        request = Request('%s://localhost:%s' + \\
+            '/server/%s/client_connect')
+        request.add_header('Content-Type', 'application/json')
+        response = urlopen(request, json.dumps({
+            'org_id': org,
+            'user_id': common_name,
+        }).encode('utf-8'))
+        response = json.loads(response.read().decode('utf-8'))
+
+        if response['client_conf']:
+            with open(sys.argv[1], 'w') as client_conf_file:
+                client_conf_file.write(response['client_conf'])
+    except HTTPError as error:
+        log_write('[FAILED] Server returned error: %%s - %%s' %% (
+            error.code, error.reason))
+        exit(1)
+    except SocketError:
+        log_write('[FAILED] Server returned socket error')
+        exit(1)
+except SystemExit:
+    raise
+except:
+    log_write('[EXCEPTION] ' + traceback.format_exc())
+    raise
+
+exit(0)
+"""
+
+# Script will run in python 2 and 3
+CLIENT_DISCONNECT_SCRIPT = """#!/usr/bin/env python
+import os
+import sys
+import json
+import time
+import traceback
+
+VALID_CHARS = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789='
+auth_log_path = '%s'
+def log_write(line):
+    with open(auth_log_path, 'a') as auth_log_file:
+        auth_log_file.write('[CLIENT_DISCONNECT][TIME=%%s]%%s\\n' %% (
+            int(time.time()), line.rstrip('\\n')))
+
+try:
+    try:
+        from urllib2 import urlopen
+    except ImportError:
+        from urllib.request import urlopen
+    try:
+        from urllib2 import Request
+    except ImportError:
+        from urllib.request import Request
+    try:
+        from urllib2 import HTTPError
+    except ImportError:
+        from urllib.error import HTTPError
+    try:
+        from socket import error as SocketError
+    except ImportError:
+        SocketError = ConnectionResetError
+
+    # Get org and common_name from environ
+    tls_env = os.environ.get('tls_id_0')
+    if not tls_env:
+        log_write('[FAILED] Missing organization or user id from environ')
+        raise AttributeError('Missing organization or user id from environ')
+    tls_env = ''.join(x for x in tls_env if x in VALID_CHARS)
+    o_index = tls_env.find('O=')
+    cn_index = tls_env.find('CN=')
+    if o_index < 0 or cn_index < 0:
+        log_write('[FAILED] Missing organization or user id from environ')
+        raise AttributeError('Missing organization or user id from environ')
+    if o_index > cn_index:
+        org = tls_env[o_index + 2:]
+        common_name = tls_env[3:o_index]
+    else:
+        org = tls_env[2:cn_index]
+        common_name = tls_env[cn_index + 3:]
+    if not org or not common_name:
+        log_write('[FAILED] Missing organization or user id from environ')
+        raise AttributeError('Missing organization or user id from environ')
+
+    try:
+        request = Request('%s://localhost:%s' + \\
+            '/server/%s/client_disconnect')
+        request.add_header('Content-Type', 'application/json')
+        response = urlopen(request, json.dumps({
+            'org_id': org,
+            'user_id': common_name,
+        }).encode('utf-8'))
+    except HTTPError as error:
+        log_write('[FAILED] Server returned error: %%s - %%s' %% (
+            error.code, error.reason))
+        exit(1)
+    except SocketError:
+        log_write('[FAILED] Server returned socket error')
+        exit(1)
+except SystemExit:
+    raise
+except:
+    log_write('[EXCEPTION] ' + traceback.format_exc())
+    raise
+
+exit(0)
+"""
