@@ -1,4 +1,5 @@
 from pritunl.constants import *
+from pritunl.exceptions import *
 from pritunl.server import Server
 from pritunl.node_server import NodeServer
 from pritunl.organization import Organization
@@ -28,12 +29,24 @@ def _port_invalid():
             'error_msg': PORT_INVALID_MSG,
         }, 400)
 
+def _dh_param_bits_invalid():
+        return utils.jsonify({
+            'error': DH_PARAM_BITS_INVALID,
+            'error_msg': DH_PARAM_BITS_INVALID_MSG,
+        }, 400)
+
 def _local_network_invalid():
     return utils.jsonify({
         'error': LOCAL_NETWORK_INVALID,
         'error_msg': LOCAL_NETWORK_INVALID_MSG,
     }, 400)
 
+def _dns_server_invalid():
+    return utils.jsonify({
+        'error': DNS_SERVER_INVALID,
+        'error_msg': DNS_SERVER_INVALID_MSG,
+    }, 400)
+
 @app_server.app.route('/server', methods=['GET'])
 @app_server.auth
 def server_get():
@@ -69,8 +82,7 @@ def server_put_post(server_id=None):
     name_def = False
     if 'name' in flask.request.json:
         name_def = True
-        name = flask.request.json['name']
-        name = ''.join(x for x in name if x.isalnum() or x in NAME_SAFE_CHARS)
+        name = utils.filter_str(flask.request.json['name'])
 
     type = SERVER_NAME
     type_def = False
@@ -143,7 +155,7 @@ def server_put_post(server_id=None):
         protocol_def = True
         protocol = flask.request.json['protocol'].lower()
 
-        if protocol not in {'udp', 'tcp'}:
+        if protocol not in ('udp', 'tcp'):
             return utils.jsonify({
                 'error': PROTOCOL_INVALID,
                 'error_msg': PROTOCOL_INVALID_MSG,
@@ -163,11 +175,25 @@ def server_put_post(server_id=None):
         if port < 1 or port > 65535:
             return _port_invalid()
 
+    dh_param_bits = None
+    dh_param_bits_def = False
+    if 'dh_param_bits' in flask.request.json:
+        dh_param_bits_def = True
+        dh_param_bits = flask.request.json['dh_param_bits']
+
+        try:
+            dh_param_bits = int(dh_param_bits)
+        except ValueError:
+            return _dh_param_bits_invalid()
+
+        if dh_param_bits not in {1024, 1536, 2048, 3072, 4096}:
+            return _dh_param_bits_invalid()
+
     local_networks = None
     local_networks_def = False
     if 'local_networks' in flask.request.json:
         local_networks_def = True
-        local_networks = flask.request.json['local_networks']
+        local_networks = flask.request.json['local_networks'] or []
 
         for local_network in local_networks:
             local_network_split = local_network.split('/')
@@ -196,13 +222,21 @@ def server_put_post(server_id=None):
             if subnet < 8 or subnet > 30:
                 return _local_network_invalid()
 
+    dns_servers = None
+    dns_servers_def = False
+    if 'dns_servers' in flask.request.json:
+        dns_servers_def = True
+        dns_servers = flask.request.json['dns_servers'] or []
+
+        for dns_server in dns_servers:
+            if not re.match(IP_REGEX, dns_server):
+                return _dns_server_invalid()
+
     public_address = None
     public_address_def = False
     if 'public_address' in flask.request.json:
         public_address_def = True
-        public_address = flask.request.json['public_address']
-        public_address = ''.join(
-            x for x in public_address if x.isalnum() or x == '.')
+        public_address = utils.filter_str(flask.request.json['public_address'])
 
     debug = False
     debug_def = False
@@ -223,11 +257,11 @@ def server_put_post(server_id=None):
         lzo_compression = True if flask.request.json[
             'lzo_compression'] else False
 
-    node_ip = None
-    node_ip_def = False
-    if 'node_ip' in flask.request.json:
-        node_ip_def = True
-        node_ip = flask.request.json['node_ip']
+    node_host = None
+    node_host_def = False
+    if 'node_host' in flask.request.json:
+        node_host_def = True
+        node_host = utils.filter_str(flask.request.json['node_host'])
 
     node_port = None
     node_port_def = False
@@ -248,7 +282,7 @@ def server_put_post(server_id=None):
     node_key_def = False
     if 'node_key' in flask.request.json:
         node_key_def = True
-        node_key = flask.request.json['node_key']
+        node_key = utils.filter_str(flask.request.json['node_key'])
 
     if not server_id:
         if not name_def:
@@ -258,6 +292,7 @@ def server_put_post(server_id=None):
             }, 400)
 
         if not network_def:
+            network_def = True
             for i in xrange(5000):
                 rand_network = '10.%s.%s.0/24' % (
                     random.randint(15,250), random.randint(15,250))
@@ -271,6 +306,7 @@ def server_put_post(server_id=None):
                 }, 400)
 
         if not interface_def:
+            interface_def = True
             for i in xrange(64):
                 rand_interface = 'tun%s' % i
                 if rand_interface not in interface_used:
@@ -283,6 +319,7 @@ def server_put_post(server_id=None):
                 }, 400)
 
         if not port_def:
+            port_def = True
             rand_ports = range(10000, 19999)
             random.shuffle(rand_ports)
             for rand_port in rand_ports:
@@ -295,11 +332,34 @@ def server_put_post(server_id=None):
                     'error_msg': PORT_PROTOCOL_IN_USE_MSG,
                 }, 400)
 
+        if not dh_param_bits_def:
+            dh_param_bits_def = True
+            dh_param_bits = DEFAULT_DH_PARAM_BITS
+
         if not public_address_def:
-            if not app_server.public_ip:
-                app_server.load_public_ip()
+            public_address_def = True
             public_address = app_server.public_ip
 
+    if network_def:
+        if network in network_used:
+            return utils.jsonify({
+                'error': NETWORK_IN_USE,
+                'error_msg': NETWORK_IN_USE_MSG,
+            }, 400)
+    if interface_def:
+        if interface in interface_used:
+            return utils.jsonify({
+                'error': INTERFACE_IN_USE,
+                'error_msg': INTERFACE_IN_USE_MSG,
+            }, 400)
+    if port_def:
+        if '%s%s' % (port, protocol) in port_used:
+            return utils.jsonify({
+                'error': PORT_PROTOCOL_IN_USE,
+                'error_msg': PORT_PROTOCOL_IN_USE_MSG,
+            }, 400)
+
+    if not server_id:
         if type == NODE_SERVER_NAME:
             server = NodeServer(
                 name=name,
@@ -307,12 +367,14 @@ def server_put_post(server_id=None):
                 interface=interface,
                 port=port,
                 protocol=protocol,
+                dh_param_bits=dh_param_bits,
                 local_networks=local_networks,
+                dns_servers=dns_servers,
                 public_address=public_address,
                 otp_auth=otp_auth,
                 lzo_compression=lzo_compression,
                 debug=debug,
-                node_ip=node_ip,
+                node_host=node_host,
                 node_port=node_port,
                 node_key=node_key,
             )
@@ -323,7 +385,9 @@ def server_put_post(server_id=None):
                 interface=interface,
                 port=port,
                 protocol=protocol,
+                dh_param_bits=dh_param_bits,
                 local_networks=local_networks,
+                dns_servers=dns_servers,
                 public_address=public_address,
                 otp_auth=otp_auth,
                 lzo_compression=lzo_compression,
@@ -339,30 +403,19 @@ def server_put_post(server_id=None):
         if name_def:
             server.name = name
         if network_def:
-            if network in network_used:
-                return utils.jsonify({
-                    'error': NETWORK_IN_USE,
-                    'error_msg': NETWORK_IN_USE_MSG,
-                }, 400)
             server.network = network
         if interface_def:
-            if interface in interface_used:
-                return utils.jsonify({
-                    'error': INTERFACE_IN_USE,
-                    'error_msg': INTERFACE_IN_USE_MSG,
-                }, 400)
             server.interface = interface
         if port_def:
-            if '%s%s' % (port, protocol) in port_used:
-                return utils.jsonify({
-                    'error': PORT_PROTOCOL_IN_USE,
-                    'error_msg': PORT_PROTOCOL_IN_USE_MSG,
-                }, 400)
             server.port = port
         if protocol_def:
             server.protocol = protocol
+        if dh_param_bits_def:
+            server.dh_param_bits = dh_param_bits
         if local_networks_def:
             server.local_networks = local_networks
+        if dns_servers_def:
+            server.dns_servers = dns_servers
         if public_address_def:
             server.public_address = public_address
         if otp_auth_def:
@@ -372,8 +425,8 @@ def server_put_post(server_id=None):
         if debug_def:
             server.debug = debug
         if server.type == NODE_SERVER_NAME:
-            if node_ip_def:
-                server.node_ip = node_ip
+            if node_host_def:
+                server.node_host = node_host
             if node_port_def:
                 server.node_port = node_port
             if node_key_def:
@@ -438,12 +491,24 @@ def server_org_delete(server_id, org_id):
 @app_server.auth
 def server_operation_put(server_id, operation):
     server = Server.get_server(id=server_id)
-    if operation == START:
-        server.start()
-    if operation == STOP:
-        server.stop()
-    elif operation == RESTART:
-        server.restart()
+
+    try:
+        if operation == START:
+            server.start()
+        if operation == STOP:
+            server.stop()
+        elif operation == RESTART:
+            server.restart()
+    except NodeConnectionError:
+        return utils.jsonify({
+            'error': NODE_CONNECTION_ERROR,
+            'error_msg': NODE_CONNECTION_ERROR_MSG,
+        }, 400)
+    except InvalidNodeAPIKey:
+        return utils.jsonify({
+            'error': NODE_API_KEY_INVLID,
+            'error_msg': NODE_API_KEY_INVLID_MSG,
+        }, 400)
 
     return utils.jsonify(server.dict())
 
@@ -463,37 +528,9 @@ def server_output_delete(server_id):
     server.clear_output()
     return utils.jsonify({})
 
-@app_server.app.route('/server/<server_id>/tls_verify', methods=['POST'])
-def server_tls_verify_get(server_id):
-    org_id = flask.request.json['org_id']
-    user_id = flask.request.json['user_id']
-
-    server = Server(server_id)
-    if not server:
-        return utils.jsonify({
-            'error': SERVER_INVALID,
-            'error_msg': SERVER_INVALID_MSG,
-        }, 401)
-    org = server.get_org(org_id)
-    if not org:
-        return utils.jsonify({
-            'error': ORG_INVALID,
-            'error_msg': ORG_INVALID_MSG,
-        }, 401)
-    user = org.get_user(user_id)
-    if not user:
-        return utils.jsonify({
-            'error': USER_INVALID,
-            'error_msg': USER_INVALID_MSG,
-        }, 401)
-
-    return utils.jsonify({
-        'authenticated': True,
-    })
-
 @app_server.app.route('/server/<server_id>/tls_verify', methods=['POST'])
 @app_server.local_only
-def server_tls_verify_get(server_id):
+def server_tls_verify_post(server_id):
     org_id = flask.request.json['org_id']
     user_id = flask.request.json['user_id']
 
@@ -526,7 +563,7 @@ def server_tls_verify_get(server_id):
 
 @app_server.app.route('/server/<server_id>/otp_verify', methods=['POST'])
 @app_server.local_only
-def server_otp_verify_get(server_id):
+def server_otp_verify_post(server_id):
     org_id = flask.request.json['org_id']
     user_id = flask.request.json['user_id']
     otp_code = flask.request.json['otp_code']
@@ -564,3 +601,60 @@ def server_otp_verify_get(server_id):
     return utils.jsonify({
         'authenticated': True,
     })
+
+@app_server.app.route('/server/<server_id>/client_connect', methods=['POST'])
+@app_server.local_only
+def server_client_connect_post(server_id):
+    org_id = flask.request.json['org_id']
+    user_id = flask.request.json['user_id']
+
+    server = Server(server_id)
+    if not server:
+        return utils.jsonify({
+            'error': SERVER_INVALID,
+            'error_msg': SERVER_INVALID_MSG,
+        }, 401)
+    org = server.get_org(org_id)
+    if not org:
+        return utils.jsonify({
+            'error': ORG_INVALID,
+            'error_msg': ORG_INVALID_MSG,
+        }, 401)
+    user = org.get_user(user_id)
+    if not user:
+        return utils.jsonify({
+            'error': USER_INVALID,
+            'error_msg': USER_INVALID_MSG,
+        }, 401)
+
+    return utils.jsonify({
+        'client_conf': None,
+    })
+
+@app_server.app.route('/server/<server_id>/client_disconnect',
+    methods=['POST'])
+@app_server.local_only
+def server_client_disconnect_post(server_id):
+    org_id = flask.request.json['org_id']
+    user_id = flask.request.json['user_id']
+
+    server = Server(server_id)
+    if not server:
+        return utils.jsonify({
+            'error': SERVER_INVALID,
+            'error_msg': SERVER_INVALID_MSG,
+        }, 401)
+    org = server.get_org(org_id)
+    if not org:
+        return utils.jsonify({
+            'error': ORG_INVALID,
+            'error_msg': ORG_INVALID_MSG,
+        }, 401)
+    user = org.get_user(user_id)
+    if not user:
+        return utils.jsonify({
+            'error': USER_INVALID,
+            'error_msg': USER_INVALID_MSG,
+        }, 401)
+
+    return utils.jsonify({})
