@@ -1,4 +1,5 @@
 from constants import *
+from exceptions import *
 from pritunl import app_server
 from server import Server
 from organization import Organization
@@ -16,7 +17,7 @@ import time
 logger = logging.getLogger(APP_NAME)
 
 class NodeServer(Server):
-    str_options = Server.str_options | {'node_ip', 'node_key'}
+    str_options = Server.str_options | {'node_host', 'node_key'}
     int_options = Server.int_options | {'node_port'}
     default_options = dict(Server.default_options.items() + {
         'node_port': 9800,
@@ -25,9 +26,11 @@ class NodeServer(Server):
 
     def dict(self):
         server_dict = Server.dict(self)
-        server_dict['node_ip'] = self.node_ip
-        server_dict['node_port'] = self.node_port
-        server_dict['node_key'] = self.node_key
+        server_dict.update({
+            'node_host': self.node_host,
+            'node_port': self.node_port,
+            'node_key': self.node_key,
+        })
         return server_dict
 
     def _initialize(self):
@@ -35,66 +38,84 @@ class NodeServer(Server):
         with open(os.path.join(self.path, NODE_SERVER_NAME), 'w'):
             pass
 
+    def _request(self, method, endpoint='', timeout=HTTP_REQUEST_TIMEOUT,
+            json_data=None):
+        return getattr(utils.request, method)(
+            self._get_node_url() + endpoint,
+            timeout=timeout,
+            headers={
+                'API-Key': self.node_key,
+            },
+            json_data=json_data,
+        )
+
     def _get_node_url(self):
-        return 'http://%s:%s/server/%s' % (
-            self.node_ip, self.node_port, self.id)
+        return 'https://%s:%s/server/%s' % (
+            self.node_host, self.node_port, self.id)
+
+    def _sub_thread(self):
+        for message in cache_db.subscribe(self.get_cache_key()):
+            try:
+                if message == 'stop':
+                    self._interrupt = True
+                elif message == 'stopped':
+                    break
+            except OSError:
+                pass
 
     def _com_thread(self):
-        responses = []
         trigger_event = False
+        responses = []
 
         try:
             while not self._interrupt and not app_server.interrupt:
-                response = utils.request.put(
-                    '%s/com' % self._get_node_url(),
-                    timeout=HTTP_REQUEST_TIMEOUT,
-                    json_data=responses,
-                )
-                if response.status_code == 200:
-                    pass
-                elif response.status_code == 410:
-                    break
-                else:
-                    logger.exception('Error with node server connection ' + \
-                        'occurred. %r' % {
-                            'server_id': self.id,
-                            'status_code': response.status_code,
-                            'reason': response.reason,
-                        })
-                    LogEntry(message='Error with node server connection ' + \
-                        'occurred "%s".' % self.name)
-                    trigger_event = True
-                    break
-                calls = response.json()
-                responses = []
-
-                for call in calls:
-                    try:
-                        responses.append({
-                            'id': call['id'],
-                            'response': getattr(self, call['command'])(
-                                *call['args']),
-                        })
-                    except:
-                        logger.exception('Node server com thread call ' + \
-                            'failed. %s' % {
+                try:
+                    response = self._request('put', endpoint='/com',
+                        timeout=HTTP_COM_REQUEST_TIMEOUT, json_data=responses)
+                    if response.status_code == 200:
+                        pass
+                    elif response.status_code == 410:
+                        break
+                    else:
+                        logger.exception('Error with node server ' + \
+                            'connection occurred. %r' % {
                                 'server_id': self.id,
-                                'call_id': call['id'],
-                                'call_command': call['command'],
-                                'call_args': call['args'],
+                                'status_code': response.status_code,
+                                'reason': response.reason,
                             })
-        except httplib.HTTPException:
-            logger.exception('Lost connection with node server. %r' % {
-                'server_id': self.id,
-            })
-            LogEntry(message='Lost connection with node server "%s".' % (
-                self.name))
-            trigger_event = True
+                        LogEntry(message='Error with node server ' + \
+                            'connection occurred "%s".' % self.name)
+                        trigger_event = True
+                        break
+                    calls = response.json()
+                    responses = []
+
+                    for call in calls:
+                        try:
+                            responses.append({
+                                'id': call['id'],
+                                'response': getattr(self, call['command'])(
+                                    *call['args']),
+                            })
+                        except:
+                            logger.exception('Node server com thread call ' + \
+                                'failed. %s' % {
+                                    'server_id': self.id,
+                                    'call_id': call['id'],
+                                    'call_command': call['command'],
+                                    'call_args': call['args'],
+                                })
+                except httplib.HTTPException:
+                    logger.exception('Lost connection with node server. %r' % {
+                        'server_id': self.id,
+                    })
+                    LogEntry(message='Lost connection with node ' + \
+                        'server "%s".' % self.name)
         finally:
             self.status = False
+            self.publish('stopped')
             if trigger_event:
                 Event(type=SERVERS_UPDATED)
-                LogEntry(message='Stopped server "%s".' % self.name)
 
     def tls_verify(self, org_id, user_id):
         org = self.get_org(org_id)
@@ -126,6 +147,32 @@ class NodeServer(Server):
             return False
         return True
 
+    def client_connect(self, org_id, user_id):
+        org = self.get_org(org_id)
+        if not org:
+            LogEntry(message='User failed authentication, ' +
+                'invalid organization "%s".' % server.name)
+            return
+        user = org.get_user(user_id)
+        if not user:
+            LogEntry(message='User failed authentication, ' +
+                'invalid user "%s".' % server.name)
+            return
+        return
+
+    def client_disconnect(self, org_id, user_id):
+        org = self.get_org(org_id)
+        if not org:
+            LogEntry(message='User failed authentication, ' +
+                'invalid organization "%s".' % server.name)
+            return
+        user = org.get_user(user_id)
+        if not user:
+            LogEntry(message='User failed authentication, ' +
+                'invalid user "%s".' % server.name)
+            return
+        return
+
     def update_clients(self, clients):
         client_count = len(self.clients)
         self.clients = clients
@@ -134,27 +181,12 @@ class NodeServer(Server):
                 Event(type=USERS_UPDATED, resource_id=org.id)
             Event(type=SERVERS_UPDATED)
 
-    def otp_verify(self, org_id, user_id, otp_code):
-        org = self.get_org(org_id)
-        if not org:
-            LogEntry(message='User failed authentication, ' +
-                'invalid organization "%s".' % server.name)
-            return False
-        user = org.get_user(user_id)
-        if not user:
-            LogEntry(message='User failed authentication, ' +
-                'invalid user "%s".' % server.name)
-            return False
-        if not user.verify_otp_code(otp_code):
-            LogEntry(message='User failed two-step authentication "%s".' % (
-                user.name))
-            return False
-        return True
-
     def _generate_ovpn_conf(self):
         if not self.org_count:
-            raise ValueError('Ovpn conf cannot be generated without ' + \
-                'any organizations')
+            raise ServerMissingOrg('Ovpn conf cannot be generated without ' + \
+                'any organizations', {
+                    'server_id': self.id,
+                })
 
         logger.debug('Generating node server ovpn conf. %r' % {
             'server_id': self.id,
@@ -170,22 +202,24 @@ class NodeServer(Server):
         primary_user = primary_org.get_user(self.primary_user)
 
         self.generate_ca_cert()
-        self._generate_tls_verify()
-        self._generate_user_pass_verify()
 
+        push = ''
         if self.local_networks:
-            push = ''
             for network in self.local_networks:
                 push += 'push "route %s %s"\n' % self._parse_network(network)
-            push = push.rstrip()
         else:
-            push = 'push "redirect-gateway"'
+            push += 'push "redirect-gateway"\n'
+        for dns_server in self.dns_servers:
+            push += 'push "dhcp-option DNS %s"\n' % dns_server
+        push = push.rstrip()
 
         server_conf = OVPN_INLINE_SERVER_CONF % (
             self.port,
             self.protocol,
             self.interface,
             '%s',
+            '%s',
+            '%s',
             '%s %s' % self._parse_network(self.network),
             '%s',
             push,
@@ -219,8 +253,10 @@ class NodeServer(Server):
         if self.status:
             return
         if not self.org_count:
-            raise ValueError('Server cannot be started without ' + \
-                'any organizations')
+            raise ServerMissingOrg('Server cannot be started without ' + \
+                'any organizations', {
+                    'server_id': self.id,
+                })
 
         logger.debug('Starting node server. %r' % {
             'server_id': self.id,
@@ -228,35 +264,38 @@ class NodeServer(Server):
         ovpn_conf = self._generate_ovpn_conf()
 
         try:
-            response = utils.request.post(
-                self._get_node_url(),
-                timeout=HTTP_REQUEST_TIMEOUT,
-                json_data={
-                    'network': self.network,
-                    'local_networks': self.local_networks,
-                    'ovpn_conf': ovpn_conf,
-                },
-            )
-        except:
-            logger.exception('Failed to start node server. %r' % {
+            response = self._request('post', json_data={
+                'interface': self.interface,
+                'network': self.network,
+                'local_networks': self.local_networks,
+                'ovpn_conf': ovpn_conf,
+                'server_ver': NODE_SERVER_VER,
+            })
+        except httplib.HTTPException:
+            raise NodeConnectionError('Failed to connect to node server', {
                 'server_id': self.id,
             })
-            raise
 
-        if response.status_code != 200:
-            logger.error('Failed to start node server. %r' % {
+        if response.status_code == 401:
+            raise InvalidNodeAPIKey('Invalid node server api key', {
+                'server_id': self.id,
+                'status_code': response.status_code,
+                'reason': response.reason,
+            })
+        elif response.status_code != 200:
+            raise ServerStartError('Failed to start node server', {
                 'server_id': self.id,
                 'status_code': response.status_code,
                 'reason': response.reason,
             })
-            raise ValueError('Node server returned error')
 
-        self._interrupt = False
         cache_db.dict_set(self.get_cache_key(), 'start_time',
             str(int(time.time() - 1)))
         self.clear_output()
-        threading.Thread(target=self._com_thread).start()
+        self._interrupt = False
         self.status = True
+        threading.Thread(target=self._com_thread).start()
+        threading.Thread(target=self._sub_thread).start()
 
         if not silent:
             Event(type=SERVERS_UPDATED)
@@ -265,27 +304,33 @@ class NodeServer(Server):
     def stop(self, silent=False):
         if not self.status:
             return
-        self._interrupt = True
 
         try:
-            response = utils.request.delete(
-                self._get_node_url(),
-                timeout=HTTP_REQUEST_TIMEOUT,
-            )
-        except:
-            logger.exception('Failed to stop node server. %r' % {
+            response = self._request('delete')
+            if response.status_code != 200:
+                raise ServerStopError('Failed to stop node server', {
+                    'server_id': self.id,
+                    'status_code': response.status_code,
+                    'reason': response.reason,
+                })
+        except httplib.HTTPException:
+            logger.exception('Failed to delete stopped node server', {
                 'server_id': self.id,
             })
-            raise
 
-        if response.status_code != 200:
-            logger.error('Failed to stop node server. %r' % {
-                'server_id': self.id,
-                'status_code': response.status_code,
-                'reason': response.reason,
-            })
-            raise ValueError('Node server returned error')
-        self.status = False
+        if self.status:
+            stopped = False
+            self.publish('stop')
+            for message in cache_db.subscribe(self.get_cache_key(),
+                    HTTP_COM_REQUEST_TIMEOUT + 3):
+                if message == 'stopped':
+                    stopped = True
+                    break
+            if not stopped:
+                raise ServerStopError('Server thread failed to return ' + \
+                    'stop event', {
+                        'server_id': self.id,
+                    })
 
         if not silent:
             Event(type=SERVERS_UPDATED)
