@@ -1,4 +1,5 @@
 from constants import *
+from exceptions import *
 from pritunl import app_server
 from config import Config
 from organization import Organization
@@ -22,17 +23,20 @@ logger = logging.getLogger(APP_NAME)
 class Server(Config):
     str_options = {'name', 'network', 'interface', 'protocol',
         'local_networks', 'public_address', 'primary_organization',
-        'primary_user', 'organizations', 'local_network'}
+        'primary_user', 'organizations', 'local_network', 'dns_servers'}
     bool_options = {'otp_auth', 'lzo_compression', 'debug'}
-    int_options = {'port'}
-    list_options = {'organizations', 'local_networks'}
+    int_options = {'port', 'dh_param_bits'}
+    list_options = {'organizations', 'local_networks', 'dns_servers'}
+    cached = True
     cache_prefix = 'server'
     type = 'server'
 
     def __init__(self, id=None, **kwargs):
         Config.__init__(self)
         self._cur_event = None
+        self._cur_client_count = 0
         self._last_event = 0
+        self._rebuild_dh_params = False
 
         if id is None:
             self.id = uuid.uuid4().hex
@@ -50,6 +54,10 @@ class Server(Config):
             TLS_VERIFY_NAME)
         self.user_pass_verify_path = os.path.join(self.path, TEMP_DIR,
             USER_PASS_VERIFY_NAME)
+        self.client_connect_path = os.path.join(self.path, TEMP_DIR,
+            CLIENT_CONNECT_NAME)
+        self.client_disconnect_path = os.path.join(self.path, TEMP_DIR,
+            CLIENT_DISCONNECT_NAME)
         self.ovpn_status_path = os.path.join(self.path, TEMP_DIR,
             OVPN_STATUS_NAME)
         self.auth_log_path = os.path.join(app_server.data_path, AUTH_LOG_NAME)
@@ -64,10 +72,14 @@ class Server(Config):
                 cache_db.dict_set(self.get_cache_key(), name, 't')
             else:
                 cache_db.dict_set(self.get_cache_key(), name, 'f')
+            return
         elif name == 'clients':
             cache_db.dict_set(self.get_cache_key(), name, json.dumps(value))
-        else:
-            Config.__setattr__(self, name, value)
+            return
+        elif name == 'dh_param_bits':
+            if not self._loaded or self.dh_param_bits != value:
+                self._rebuild_dh_params = True
+        Config.__setattr__(self, name, value)
 
     def __getattr__(self, name):
         if name == 'status':
@@ -106,7 +118,9 @@ class Server(Config):
             'interface': self.interface,
             'port': self.port,
             'protocol': self.protocol,
+            'dh_param_bits': self.dh_param_bits,
             'local_networks': self.local_networks,
+            'dns_servers': self.dns_servers,
             'public_address': self.public_address,
             'otp_auth': True if self.otp_auth else False,
             'lzo_compression': self.lzo_compression,
@@ -123,8 +137,16 @@ class Server(Config):
             self.local_network = None
             self.commit()
 
+    def _upgrade_0_10_6(self):
+        if not self.dh_param_bits:
+            logger.debug('Upgrading server to v0.10.6... %r' % {
+                'server_id': self.id,
+            })
+            self.dh_param_bits = app_server.dh_param_bits
+            self.commit()
+
     def _initialize(self):
-        logger.info('Initialize new server. %r' % {
+        logger.debug('Initialize new server. %r' % {
             'server_id': self.id,
         })
         os.makedirs(os.path.join(self.path, TEMP_DIR))
@@ -158,7 +180,7 @@ class Server(Config):
         threading.Thread(target=_target).start()
 
     def remove(self):
-        logger.info('Removing server. %r' % {
+        logger.debug('Removing server. %r' % {
             'server_id': self.id,
         })
         name = self.name
@@ -172,13 +194,17 @@ class Server(Config):
         Event(type=SERVERS_UPDATED)
 
     def commit(self):
+        if self._rebuild_dh_params:
+            self._generate_dh_param()
         Config.commit(self)
         Event(type=SERVERS_UPDATED)
 
     def _create_primary_user(self):
         if not self.org_count:
-            raise ValueError('Primary user cannot be created without ' + \
-                'any organizations')
+            raise ServerMissingOrg('Primary user cannot be created ' + \
+                'without any organizations', {
+                    'server_id': self.id,
+                })
         logger.debug('Creating primary user. %r' % {
             'server_id': self.id,
         })
@@ -298,10 +324,11 @@ class Server(Config):
         logger.debug('Generating server dh params. %r' % {
             'server_id': self.id,
         })
+        self._rebuild_dh_params = False
         args = [
             'openssl', 'dhparam',
             '-out', self.dh_param_path,
-            str(app_server.dh_param_bits)
+            str(self.dh_param_bits)
         ]
         subprocess.check_call(args, stdout=subprocess.PIPE,
             stderr=subprocess.PIPE)
@@ -325,38 +352,31 @@ class Server(Config):
                 with open(ca_path, 'r') as org_ca_cert:
                     server_ca_cert.write(org_ca_cert.read())
 
-    def _generate_tls_verify(self):
-        logger.debug('Generating tls verify script. %r' % {
+    def _generate_scripts(self):
+        logger.debug('Generating openvpn scripts. %r' % {
             'server_id': self.id,
         })
-        with open(self.tls_verify_path, 'w') as tls_verify_file:
-            os.chmod(self.tls_verify_path, 0755)
-            data_path = app_server.data_path
-            tls_verify_file.write(TLS_VERIFY_SCRIPT % (
-                self.auth_log_path,
-                app_server.web_protocol,
-                app_server.port,
-                self.id,
-            ))
-
-    def _generate_user_pass_verify(self):
-        logger.debug('Generating user pass verify script. %r' % {
-            'server_id': self.id,
-        })
-        with open(self.user_pass_verify_path, 'w') as user_pass_verify_file:
-            os.chmod(self.user_pass_verify_path, 0755)
-            data_path = app_server.data_path
-            user_pass_verify_file.write(USER_PASS_VERIFY_SCRIPT % (
-                self.auth_log_path,
-                app_server.web_protocol,
-                app_server.port,
-                self.id,
-            ))
+        for script, script_path in (
+                    (TLS_VERIFY_SCRIPT, self.tls_verify_path),
+                    (USER_PASS_VERIFY_SCRIPT, self.user_pass_verify_path),
+                    (CLIENT_CONNECT_SCRIPT, self.client_connect_path),
+                    (CLIENT_DISCONNECT_SCRIPT, self.client_disconnect_path),
+                ):
+            with open(script_path, 'w') as script_file:
+                os.chmod(script_path, 0755)
+                script_file.write(script % (
+                    self.auth_log_path,
+                    app_server.web_protocol,
+                    app_server.port,
+                    self.id,
+                ))
 
     def _generate_ovpn_conf(self, inline=False):
         if not self.org_count:
-            raise ValueError('Ovpn conf cannot be generated without ' + \
-                'any organizations')
+            raise ServerMissingOrg('Ovpn conf cannot be generated without ' + \
+                'any organizations', {
+                    'server_id': self.id,
+                })
 
         logger.debug('Generating server ovpn conf. %r' % {
             'server_id': self.id,
@@ -372,16 +392,17 @@ class Server(Config):
         primary_user = primary_org.get_user(self.primary_user)
 
         self.generate_ca_cert()
-        self._generate_tls_verify()
-        self._generate_user_pass_verify()
+        self._generate_scripts()
 
+        push = ''
         if self.local_networks:
-            push = ''
             for network in self.local_networks:
                 push += 'push "route %s %s"\n' % self._parse_network(network)
-            push = push.rstrip()
         else:
-            push = 'push "redirect-gateway"'
+            push += 'push "redirect-gateway"\n'
+        for dns_server in self.dns_servers:
+            push += 'push "dhcp-option DNS %s"\n' % dns_server
+        push = push.rstrip()
 
         if not inline:
             server_conf = OVPN_SERVER_CONF % (
@@ -392,6 +413,8 @@ class Server(Config):
                 primary_user.cert_path,
                 primary_user.key_path,
                 self.tls_verify_path,
+                self.client_connect_path,
+                self.client_disconnect_path,
                 self.dh_param_path,
                 '%s %s' % self._parse_network(self.network),
                 self.ifc_pool_path,
@@ -406,6 +429,8 @@ class Server(Config):
                 self.protocol,
                 self.interface,
                 self.tls_verify_path,
+                self.client_connect_path,
+                self.client_disconnect_path,
                 '%s %s' % self._parse_network(self.network),
                 self.ifc_pool_path,
                 push,
@@ -449,8 +474,8 @@ class Server(Config):
             })
             raise
 
-    def _generate_iptable_rules(self):
-        iptable_rules = []
+    def _generate_iptables_rules(self):
+        rules = []
 
         try:
             routes_output = utils.check_output(['route', '-n'],
@@ -469,14 +494,17 @@ class Server(Config):
             routes[line_split[0]] = line_split[7]
 
         if '0.0.0.0' not in routes:
-            logger.error('Failed to find default network interface. %r' % {
+            raise IptablesError('Failed to find default network interface', {
                 'server_id': self.id,
             })
-            raise ValueError('Failed to find default network interface')
         default_interface = routes['0.0.0.0']
 
+        rules.append(['INPUT', '-i', self.interface, '-j', 'ACCEPT'])
+        rules.append(['FORWARD', '-i', self.interface, '-j', 'ACCEPT'])
+
+        interfaces = set()
         for network_address in self.local_networks or ['0.0.0.0/0']:
-            args = []
+            args = ['POSTROUTING', '-t', 'nat']
             network = self._parse_network(network_address)[0]
 
             if network not in routes:
@@ -487,65 +515,69 @@ class Server(Config):
                 interface = default_interface
             else:
                 interface = routes[network]
+            interfaces.add(interface)
 
             if network != '0.0.0.0':
                 args += ['-d', network_address]
 
             args += ['-s', self.network, '-o', interface, '-j', 'MASQUERADE']
-            iptable_rules.append(args)
+            rules.append(args)
 
-        return iptable_rules
+        for interface in interfaces:
+            rules.append(['FORWARD', '-i', interface, '-o', self.interface,
+                '-m', 'state', '--state', 'ESTABLISHED,RELATED',
+                '-j', 'ACCEPT'])
+            rules.append(['FORWARD', '-i', self.interface, '-o', interface,
+                '-m', 'state', '--state', 'ESTABLISHED,RELATED',
+                '-j', 'ACCEPT'])
 
-    def _exists_iptable_rules(self):
-        logger.debug('Checking for iptable rules. %r' % {
+        return rules
+
+    def _exists_iptables_rules(self, rule):
+        logger.debug('Checking for iptables rule. %r' % {
             'server_id': self.id,
+            'rule': rule,
         })
-        for iptable_rule in self._generate_iptable_rules():
-            try:
-                subprocess.check_call(['iptables', '-t', 'nat', '-C',
-                    'POSTROUTING'] + iptable_rule,
-                    stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-            except subprocess.CalledProcessError:
-                return False
+        try:
+            subprocess.check_call(['iptables', '-C'] + rule,
+                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        except subprocess.CalledProcessError:
+            return False
         return True
 
-    def _set_iptable_rules(self):
-        if self._exists_iptable_rules():
-            return
-
-        logger.debug('Setting iptable rules. %r' % {
+    def _set_iptables_rules(self):
+        logger.debug('Setting iptables rules. %r' % {
             'server_id': self.id,
         })
-        for iptable_rule in self._generate_iptable_rules():
-            try:
-                subprocess.check_call(['iptables', '-t', 'nat', '-A',
-                    'POSTROUTING'] + iptable_rule,
-                    stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-            except subprocess.CalledProcessError:
-                logger.exception('Failed to apply iptables ' + \
-                    'routing rules. %r' % {
-                        'server_id': self.id,
-                    })
-                raise
+        for rule in self._generate_iptables_rules():
+            if not self._exists_iptables_rules(rule):
+                try:
+                    subprocess.check_call(['iptables', '-A'] + rule,
+                        stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+                except subprocess.CalledProcessError:
+                    logger.exception('Failed to apply iptables ' + \
+                        'routing rule. %r' % {
+                            'server_id': self.id,
+                            'rule': rule,
+                        })
+                    raise
 
-    def _clear_iptable_rules(self):
-        if not self._exists_iptable_rules():
-            return
-        logger.debug('Clearing iptable rules. %r' % {
+    def _clear_iptables_rules(self):
+        logger.debug('Clearing iptables rules. %r' % {
             'server_id': self.id,
         })
-
-        for iptable_rule in self._generate_iptable_rules():
-            try:
-                subprocess.check_call(['iptables', '-t', 'nat', '-D',
-                    'POSTROUTING'] + iptable_rule,
-                    stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-            except subprocess.CalledProcessError:
-                logger.exception('Failed to clear iptables ' + \
-                    'routing rules. %r' % {
-                        'server_id': self.id,
-                    })
-                raise
+        for rule in self._generate_iptables_rules():
+            if self._exists_iptables_rules(rule):
+                try:
+                    subprocess.check_call(['iptables', '-D'] + rule,
+                        stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+                except subprocess.CalledProcessError:
+                    logger.exception('Failed to clear iptables ' + \
+                        'routing rule. %r' % {
+                            'server_id': self.id,
+                            'rule': rule,
+                        })
+                    raise
 
     def _sub_thread(self, process):
         for message in cache_db.subscribe(self.get_cache_key()):
@@ -566,16 +598,11 @@ class Server(Config):
             # Check interrupt every 0.1s check client count every 1s
             if i == 9:
                 i = 0
-                client_count = len(self.update_clients())
-                if client_count != cur_client_count:
-                    cur_client_count = client_count
-                    for org in self.iter_orgs():
-                        Event(type=USERS_UPDATED, resource_id=org.id)
-                    Event(type=SERVERS_UPDATED)
+                self.update_clients()
             else:
                 i += 1
             time.sleep(0.1)
-        self._clear_iptable_rules()
+        self._clear_iptables_rules()
 
     def _run_thread(self):
         logger.debug('Starting ovpn process. %r' % {
@@ -591,7 +618,7 @@ class Server(Config):
                 logger.exception('Failed to start ovpn process. %r' % {
                     'server_id': self.id,
                 })
-                cache_db.publish(self.get_cache_key(), 'stopped')
+                self.publish('stopped')
                 return
             cache_db.dict_set(self.get_cache_key(), 'start_time',
                 str(int(time.time() - 1)))
@@ -601,7 +628,7 @@ class Server(Config):
             status_thread = threading.Thread(target=self._status_thread)
             status_thread.start()
             self.status = True
-            cache_db.publish(self.get_cache_key(), 'started')
+            self.publish('started')
 
             while True:
                 line = process.stdout.readline()
@@ -616,7 +643,7 @@ class Server(Config):
             status_thread.join()
 
             self.status = False
-            cache_db.publish(self.get_cache_key(), 'stopped')
+            self.publish('stopped')
             if process.returncode != 0:
                 Event(type=SERVERS_UPDATED)
                 LogEntry(message='Server stopped unexpectedly "%s".' % (
@@ -627,20 +654,26 @@ class Server(Config):
             })
         except:
             self._interrupt = True
-            cache_db.publish(self.get_cache_key(), 'stopped')
+            self.publish('stopped')
+            raise
+
+    def publish(self, message):
+        cache_db.publish(self.get_cache_key(), message)
 
     def start(self, silent=False):
         if self.status:
             return
         if not self.org_count:
-            raise ValueError('Server cannot be started without ' + \
-                'any organizations')
+            raise ServerMissingOrg('Server cannot be started without ' + \
+                'any organizations', {
+                    'server_id': self.id,
+                })
         logger.debug('Starting server. %r' % {
             'server_id': self.id,
         })
         self._generate_ovpn_conf()
         self._enable_ip_forwarding()
-        self._set_iptable_rules()
+        self._set_iptables_rules()
         self.clear_output()
 
         threading.Thread(target=self._run_thread).start()
@@ -652,9 +685,14 @@ class Server(Config):
                 started = True
                 break
             elif message == 'stopped':
-                raise ValueError('Server failed to start')
+                raise ServerStartError('Server failed to start', {
+                    'server_id': self.id,
+                })
         if not started:
-            raise ValueError('Server thread failed to return start event')
+            raise ServerStartError('Server thread failed to return ' + \
+                'start event', {
+                    'server_id': self.id,
+                })
 
         if not silent:
             Event(type=SERVERS_UPDATED)
@@ -668,14 +706,17 @@ class Server(Config):
         })
 
         stopped = False
-        cache_db.publish(self.get_cache_key(), 'stop')
+        self.publish('stop')
         for message in cache_db.subscribe(self.get_cache_key(),
                 SUB_RESPONSE_TIMEOUT):
             if message == 'stopped':
                 stopped = True
                 break
         if not stopped:
-            raise ValueError('Server thread failed to return stop event')
+            raise ServerStopError('Server thread failed to return ' + \
+                'stop event', {
+                    'server_id': self.id,
+                })
 
         if not silent:
             Event(type=SERVERS_UPDATED)
@@ -689,7 +730,7 @@ class Server(Config):
         })
 
         stopped = False
-        cache_db.publish(self.get_cache_key(), 'stop')
+        self.publish('stop')
         for message in cache_db.subscribe(self.get_cache_key(), 2):
             if message == 'stopped':
                 stopped = True
@@ -697,7 +738,7 @@ class Server(Config):
 
         if not stopped:
             stopped = False
-            cache_db.publish(self.get_cache_key(), 'force_stop')
+            self.publish('force_stop')
             for message in cache_db.subscribe(self.get_cache_key(),
                     SUB_RESPONSE_TIMEOUT):
                 if message == 'stopped':
@@ -705,7 +746,10 @@ class Server(Config):
                     break
 
             if not stopped:
-                raise ValueError('Server thread failed to return stop event')
+                raise ServerStopError('Server thread failed to return ' + \
+                    'stop event', {
+                        'server_id': self.id,
+                    })
 
         if not silent:
             Event(type=SERVERS_UPDATED)
@@ -759,6 +803,13 @@ class Server(Config):
                         'connected_since': connected_since,
                     }
 
+        client_count = len(clients)
+        if client_count != self._cur_client_count:
+            self._cur_client_count = client_count
+            for org in self.iter_orgs():
+                Event(type=USERS_UPDATED, resource_id=org.id)
+            Event(type=SERVERS_UPDATED)
+
         self.clients = clients
         return clients
 
