@@ -21,6 +21,7 @@ ENABLE_STRESS_TESTS = False
 THREADED_STRESS_TEST = True
 UUID_RE = r'^[a-z0-9]+$'
 AUTH_HANDLERS = [
+    ('PUT', '/auth'),
     ('GET', '/export'),
     ('GET', '/event'),
     ('GET', '/key/0/0.tar'),
@@ -30,7 +31,6 @@ AUTH_HANDLERS = [
     ('POST', '/organization'),
     ('PUT', '/organization/0'),
     ('DELETE', '/organization/0'),
-    ('PUT', '/password'),
     ('GET', '/server'),
     ('POST', '/server'),
     ('PUT', '/server/0'),
@@ -91,7 +91,7 @@ requests.api.request = request
 class Session:
     def __init__(self):
         self._session = requests.Session()
-        self.response = self.post('/auth', json_data={
+        self.response = self.post('/auth/session', json_data={
             'username': USERNAME,
             'password': PASSWORD,
         })
@@ -140,32 +140,39 @@ class SessionTestCase(unittest.TestCase):
     def _clean_test_data(self):
         response = self.session.get('/organization')
         self.assertEqual(response.status_code, 200)
+
         data = response.json()
         for org in data:
-            if org['name'] in {TEST_ORG_NAME + '2', TEST_ORG_NAME + '3'}:
+            if org['name'] in (TEST_ORG_NAME + '2', TEST_ORG_NAME + '3'):
                 response = self.session.delete('/organization/%s' % org['id'])
                 self.assertEqual(response.status_code, 200)
 
+
         response = self.session.get('/user/%s' % self.org_id)
         self.assertEqual(response.status_code, 200)
+
         data = response.json()
         for user in data:
-            if user['name'] == {TEST_USER_NAME + '2', TEST_USER_NAME + '3'}:
+            if user['name'] in (TEST_USER_NAME + '2', TEST_USER_NAME + '3'):
                 response = self.session.delete('/user/%s/%s' % (
                     self.org_id, user['id']))
                 self.assertEqual(response.status_code, 200)
 
+
         response = self.session.get('/server')
         self.assertEqual(response.status_code, 200)
+
         data = response.json()
         for server in data:
-            if server['name'] == {TEST_SERVER_NAME + '2',
-                    TEST_SERVER_NAME + '3'}:
+            if server['name'] in (TEST_SERVER_NAME + '2',
+                    TEST_SERVER_NAME + '3'):
                 response = self.session.delete('/server/%s' % server['id'])
                 self.assertEqual(response.status_code, 200)
 
+
         response = self.session.get('/server/%s/organization' % self.server_id)
         self.assertEqual(response.status_code, 200)
+
         data = response.json()
         for server_org in data:
             response = self.session.delete('/server/%s/organization/%s' % (
@@ -176,55 +183,67 @@ class SessionTestCase(unittest.TestCase):
         if not self.org_id:
             response = self.session.get('/organization')
             self.assertEqual(response.status_code, 200)
+
             data = response.json()
             for org in data:
                 if org['name'] == TEST_ORG_NAME:
                     self.org_id = org['id']
 
+
         if not self.org_id:
             response = self.session.post('/organization', json_data={
                 'name': TEST_ORG_NAME,
             })
             self.assertEqual(response.status_code, 200)
+
             data = response.json()
             self.assertIn('id', data)
             self.assertIn('name', data)
             self.assertEqual(data['name'], TEST_ORG_NAME)
             self.org_id = data['id']
 
+
         if not self.user_id:
             response = self.session.get('/user/%s' % self.org_id)
             self.assertEqual(response.status_code, 200)
+
             data = response.json()
             for user in data:
                 if user['name'] == TEST_USER_NAME:
                     self.user_id = user['id']
 
+
         if not self.user_id:
             response = self.session.post('/user/%s' % self.org_id, json_data={
                 'name': TEST_USER_NAME,
             })
             self.assertEqual(response.status_code, 200)
+
             data = response.json()
             self.assertIn('id', data)
             self.assertIn('name', data)
             self.assertEqual(data['name'], TEST_USER_NAME)
             self.user_id = data['id']
 
+
         if not self.server_id:
             response = self.session.get('/server')
             self.assertEqual(response.status_code, 200)
+
             data = response.json()
             for server in data:
                 if server['name'] == TEST_SERVER_NAME:
                     self.server_id = server['id']
 
+
         if not self.server_id:
             response = self.session.post('/server', json_data={
                 'name': TEST_SERVER_NAME,
+                'dh_param_bits': 1024,
                 'otp_auth': True,
             })
             self.assertEqual(response.status_code, 200)
+
             data = response.json()
             self.assertIn('id', data)
             self.assertIn('name', data)
@@ -238,15 +257,48 @@ class SessionTestCase(unittest.TestCase):
 
 class Auth(SessionTestCase):
     @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
-    def test_auth_get(self):
-        response = requests.get('/auth')
+    def test_auth_put(self):
+        response = self.session.put('/auth', json_data={
+            'username': USERNAME,
+            'password': TEST_PASSWORD,
+        })
+        self.assertEqual(response.status_code, 200)
+
+        data = response.json()
+        self.assertIn('username', data)
+        self.assertEqual(data['username'], USERNAME)
+
+
+        response = self.session.post('/auth/token', json_data={
+            'username': USERNAME,
+            'password': TEST_PASSWORD,
+        })
+        self.assertEqual(response.status_code, 200)
+
+
+        response = self.session.put('/auth', json_data={
+            'username': USERNAME,
+            'password': PASSWORD,
+        })
+        self.assertEqual(response.status_code, 200)
+
+        data = response.json()
+        self.assertIn('username', data)
+        self.assertEqual(data['username'], USERNAME)
+
+    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
+    def test_auth_session_get(self):
+        response = requests.get('/auth/session')
+
         data = response.json()
         self.assertEqual(response.status_code, 200)
         self.assertIn('authenticated', data)
         self.assertFalse(data['authenticated'])
+        self.assertIn('username', data)
+        self.assertEqual(data['username'], USERNAME)
 
     @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
-    def test_auth_post(self):
+    def test_auth_session_delete(self):
         session = Session()
         data = session.response.json()
         self.assertEqual(session.response.status_code, 200)
@@ -254,7 +306,7 @@ class Auth(SessionTestCase):
         self.assertTrue(data['authenticated'])
 
 
-        response = session.get('/auth')
+        response = session.get('/auth/session')
         self.assertEqual(response.status_code, 200)
 
         data = response.json()
@@ -262,7 +314,7 @@ class Auth(SessionTestCase):
         self.assertTrue(data['authenticated'])
 
 
-        response = session.delete('/auth')
+        response = session.delete('/auth/session')
         self.assertEqual(response.status_code, 200)
 
         data = response.json()
@@ -270,7 +322,7 @@ class Auth(SessionTestCase):
         self.assertFalse(data['authenticated'])
 
 
-        response = session.get('/auth')
+        response = session.get('/auth/session')
         self.assertEqual(response.status_code, 200)
 
         data = response.json()
@@ -279,12 +331,12 @@ class Auth(SessionTestCase):
 
     @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
     def test_auth_post_error(self):
-        for endpoint in ['/auth', '/auth/token']:
-            for username, password in [
+        for endpoint in ('/auth/session', '/auth/token'):
+            for username, password in (
                         ('admin', 'test'),
                         ('test', 'admin'),
                         ('test', 'test'),
-                    ]:
+                    ):
                 response = requests.post(endpoint, json_data={
                     'username': username,
                     'password': password,
@@ -304,7 +356,7 @@ class Auth(SessionTestCase):
 
     @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
     def test_auth_token_get_post_delete(self):
-        response = requests.get('/auth')
+        response = requests.get('/auth/session')
         self.assertEqual(response.status_code, 200)
 
         data = response.json()
@@ -324,7 +376,7 @@ class Auth(SessionTestCase):
         auth_token = data['auth_token']
 
 
-        response = requests.get('/auth', headers={
+        response = requests.get('/auth/session', headers={
             'Auth-Token': auth_token,
         })
         self.assertEqual(response.status_code, 200)
@@ -340,7 +392,7 @@ class Auth(SessionTestCase):
         self.assertEqual(data, {})
 
 
-        response = requests.get('/auth', headers={
+        response = requests.get('/auth/session', headers={
             'Auth-Token': auth_token,
         })
         self.assertEqual(response.status_code, 200)
@@ -349,10 +401,11 @@ class Auth(SessionTestCase):
         self.assertIn('authenticated', data)
         self.assertFalse(data['authenticated'])
 
+
 class Data(SessionTestCase):
     @unittest.skipUnless(ENABLE_EXTENDED_TESTS, 'Skipping test')
     def test_export_get(self):
-        for endpoint in ['/export', '/export/pritunl.tar']:
+        for endpoint in ('/export', '/export/pritunl.tar'):
             response = self.session.get(endpoint)
             self.assertEqual(response.status_code, 200)
 
@@ -547,26 +600,6 @@ class Org(SessionTestCase):
             self.assertNotEqual(org['name'], TEST_ORG_NAME + '3')
 
 
-class Password(SessionTestCase):
-    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
-    def test_password_post(self):
-        response = self.session.put('/password', json_data={
-            'password': TEST_PASSWORD,
-        })
-        self.assertEqual(response.status_code, 200)
-
-        response = self.session.post('/auth/token', json_data={
-            'username': USERNAME,
-            'password': TEST_PASSWORD,
-        })
-        self.assertEqual(response.status_code, 200)
-
-        response = self.session.put('/password', json_data={
-            'password': PASSWORD,
-        })
-        self.assertEqual(response.status_code, 200)
-
-
 class Server(SessionTestCase):
     @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
     def test_server_post_put_get_delete(self):
@@ -576,7 +609,9 @@ class Server(SessionTestCase):
             'interface': 'tun64',
             'port': 12345,
             'protocol': 'udp',
-            'local_networks': [],
+            'dh_param_bits': 1024,
+            'local_networks': ['10.0.0.0/24'],
+            'dns_servers': ['8.8.4.4'],
             'public_address': '8.8.8.8',
             'debug': True,
             'otp_auth': False,
@@ -597,8 +632,12 @@ class Server(SessionTestCase):
         self.assertEqual(data['port'], 12345)
         self.assertIn('protocol', data)
         self.assertEqual(data['protocol'], 'udp')
+        self.assertIn('dh_param_bits', data)
+        self.assertEqual(data['dh_param_bits'], 1024)
         self.assertIn('local_networks', data)
-        self.assertEqual(data['local_networks'], [])
+        self.assertEqual(data['local_networks'], ['10.0.0.0/24'])
+        self.assertIn('dns_servers', data)
+        self.assertEqual(data['dns_servers'], ['8.8.4.4'])
         self.assertIn('public_address', data)
         self.assertEqual(data['public_address'], '8.8.8.8')
         self.assertIn('debug', data)
@@ -628,8 +667,12 @@ class Server(SessionTestCase):
         self.assertEqual(data['port'], 12345)
         self.assertIn('protocol', data)
         self.assertEqual(data['protocol'], 'udp')
+        self.assertIn('dh_param_bits', data)
+        self.assertEqual(data['dh_param_bits'], 1024)
         self.assertIn('local_networks', data)
-        self.assertEqual(data['local_networks'], [])
+        self.assertEqual(data['local_networks'], ['10.0.0.0/24'])
+        self.assertIn('dns_servers', data)
+        self.assertEqual(data['dns_servers'], ['8.8.4.4'])
         self.assertIn('public_address', data)
         self.assertEqual(data['public_address'], '8.8.8.8')
         self.assertIn('debug', data)
@@ -654,7 +697,9 @@ class Server(SessionTestCase):
             self.assertIn('interface', server)
             self.assertIn('port', server)
             self.assertIn('protocol', server)
+            self.assertIn('dh_param_bits', server)
             self.assertIn('local_networks', server)
+            self.assertIn('dns_servers', server)
             self.assertIn('public_address', server)
             self.assertIn('debug', server)
             self.assertIn('otp_auth', server)
@@ -667,7 +712,9 @@ class Server(SessionTestCase):
                 self.assertEqual(server['interface'], 'tun64')
                 self.assertEqual(server['port'], 12345)
                 self.assertEqual(server['protocol'], 'udp')
-                self.assertEqual(server['local_networks'], [])
+                self.assertEqual(server['dh_param_bits'], 1024)
+                self.assertEqual(server['local_networks'], ['10.0.0.0/24'])
+                self.assertEqual(server['dns_servers'], ['8.8.4.4'])
                 self.assertEqual(server['public_address'], '8.8.8.8')
                 self.assertTrue(server['debug'])
                 self.assertFalse(server['otp_auth'])
@@ -854,14 +901,14 @@ class Server(SessionTestCase):
         self.assertEqual(response.status_code, 200)
 
 
-        for test_network in [
+        for test_network in (
                     '10.254.254.024',
                     '10254.254.0/24',
                     '10a.254.254.0/24',
                     '11.254.254.0/24',
                     '10.255.254.1/24',
                     '10.254.254.0/24a',
-                ]:
+                ):
             response = self.session.post('/server', json_data={
                 'name': TEST_SERVER_NAME + '_test',
                 'network': test_network,
@@ -873,12 +920,13 @@ class Server(SessionTestCase):
             self.assertEqual(data['error'], 'network_invalid')
             self.assertIn('error_msg', data)
 
-        for test_interface in [
+
+        for test_interface in (
                     'tun-1',
                     'tun.0',
                     'tun65',
                     'tuna',
-                ]:
+                ):
             response = self.session.post('/server', json_data={
                 'name': TEST_SERVER_NAME + '_test',
                 'interface': test_interface,
@@ -891,10 +939,7 @@ class Server(SessionTestCase):
             self.assertIn('error_msg', data)
 
 
-        for test_port in [
-                    0,
-                    65536,
-                ]:
+        for test_port in (0, 65536):
             response = self.session.post('/server', json_data={
                 'name': TEST_SERVER_NAME + '_test',
                 'port': test_port,
@@ -907,6 +952,30 @@ class Server(SessionTestCase):
             self.assertIn('error_msg', data)
 
 
+        response = self.session.post('/server', json_data={
+            'name': TEST_SERVER_NAME + '_test',
+            'dh_param_bits': 512,
+        })
+        self.assertEqual(response.status_code, 400)
+
+        data = response.json()
+        self.assertIn('error', data)
+        self.assertEqual(data['error'], 'dh_param_bits_invalid')
+        self.assertIn('error_msg', data)
+
+
+        response = self.session.post('/server', json_data={
+            'name': TEST_SERVER_NAME + '_test',
+            'dns_servers': ['8.8.8.a'],
+        })
+        self.assertEqual(response.status_code, 400)
+
+        data = response.json()
+        self.assertIn('error', data)
+        self.assertEqual(data['error'], 'dns_server_invalid')
+        self.assertIn('error_msg', data)
+
+
         response = self.session.post('/server', json_data={
             'name': TEST_SERVER_NAME + '_test',
             'protocol': 'a',
