@@ -97,6 +97,9 @@ define([], function() {
       lzo_compression: false,
       interface: 'tun0',
       local_networks: [],
+      dns_servers: [
+        '8.8.4.4'
+      ],
       name: 'server1',
       type: 'server',
       network: '10.139.17.0/24',
@@ -107,6 +110,7 @@ define([], function() {
       output: 'Thu Oct  3 18:07:06 2013 OpenVPN 2.3.2 x86_64-unknown-linux-gnu [SSL (OpenSSL)] [LZO] [EPOLL] [eurephia] [MH] [IPv6] built on Jun  7 2013\nThu Oct  3 18:07:06 2013 NOTE: the current --script-security setting may allow this configuration to call user-defined scripts\nThu Oct  3 18:07:06 2013 TUN/TAP device tun0 opened\nThu Oct  3 18:07:06 2013 do_ifconfig, tt->ipv6=0, tt->did_ifconfig_ipv6_setup=0\nThu Oct  3 18:07:06 2013 /usr/bin/ip link set dev tun0 up mtu 1500\nThu Oct  3 18:07:06 2013 /usr/bin/ip addr add dev tun0 local 10.100.68.1 peer 10.100.68.2\nThu Oct  3 18:07:06 2013 UDPv4 link local (bound): [undef]\nThu Oct  3 18:07:06 2013 UDPv4 link remote: [undef]\nThu Oct  3 18:07:06 2013 Initialization Sequence Completed\n',
       port: '16070',
       protocol: 'udp',
+      dh_param_bits: 2048,
       public_address: '8.8.8.8',
       status: true
     },
@@ -117,6 +121,9 @@ define([], function() {
       lzo_compression: false,
       interface: 'tun1',
       local_networks: [],
+      dns_servers: [
+        '8.8.4.4'
+      ],
       name: 'server2',
       type: 'server',
       network: '10.113.56.0/24',
@@ -127,6 +134,7 @@ define([], function() {
       output: 'Thu Oct  3 18:07:06 2013 OpenVPN 2.3.2 x86_64-unknown-linux-gnu [SSL (OpenSSL)] [LZO] [EPOLL] [eurephia] [MH] [IPv6] built on Jun  7 2013\nThu Oct  3 18:07:06 2013 NOTE: the current --script-security setting may allow this configuration to call user-defined scripts\nThu Oct  3 18:07:06 2013 TUN/TAP device tun0 opened\nThu Oct  3 18:07:06 2013 do_ifconfig, tt->ipv6=0, tt->did_ifconfig_ipv6_setup=0\nThu Oct  3 18:07:06 2013 /usr/bin/ip link set dev tun0 up mtu 1500\nThu Oct  3 18:07:06 2013 /usr/bin/ip addr add dev tun0 local 10.100.68.1 peer 10.100.68.2\nThu Oct  3 18:07:06 2013 UDPv4 link local (bound): [undef]\nThu Oct  3 18:07:06 2013 UDPv4 link remote: [undef]\nThu Oct  3 18:07:06 2013 Initialization Sequence Completed\n',
       port: '9430',
       protocol: 'udp',
+      dh_param_bits: 2048,
       public_address: '8.8.8.8',
       status: true
     }
