@@ -61,7 +61,10 @@ define([
     _removeItem: function(view, slide) {
       this._hideItem(view, slide, (this.removeItem).bind(this));
     },
-    _onReset: function(collection) {
+    _onReset: function(collection, options) {
+      if (this.ignore(options)) {
+        return;
+      }
       var i;
       var newIndex;
       var modelView;
@@ -301,6 +304,8 @@ define([
         }.bind(this)
       });
     },
+    ignore: function() {
+    },
     getOptions: function() {
     },
     removeItem: function(view) {
