@@ -89,7 +89,7 @@ define([
       this.$('.server-public-address .status-text').text(
         this.model.get('public_address'));
       this.$('.server-node-host .status-text').text(
-        this.model.get('node_ip'));
+        this.model.get('node_host'));
 
       if (!this.model.get('org_count')) {
         this.$('.server-stop').hide();
@@ -206,14 +206,24 @@ define([
         success: function() {
           $(evt.target).removeAttr('disabled');
         }.bind(this),
-        error: function() {
+        error: function(model, response) {
+          var alertView;
           $(evt.target).removeAttr('disabled');
-          var alertView = new AlertView({
-            type: 'danger',
-            message: 'Failed to ' + operation +
-              ' the server, server error occurred.',
-            dismissable: true
-          });
+          if (response.responseJSON) {
+            alertView = new AlertView({
+              type: 'danger',
+              message: response.responseJSON.error_msg,
+              dismissable: true
+            });
+          }
+          else {
+            alertView = new AlertView({
+              type: 'danger',
+              message: 'Failed to ' + operation +
+                ' the server, server error occurred.',
+              dismissable: true
+            });
+          }
           $('.alerts-container').append(alertView.render().el);
           this.addView(alertView);
         }.bind(this)
